@@ -820,9 +820,9 @@ do_comp_vars(int test, int na, char *sa, int nb, char *sb, int mod)
 		    na = -na;
 		    add = -1;
 		} else {
-		    p = compprefix + 1;
-		    if (*p == Meta)
-			p++;
+		    p = compprefix + 1 + (*compprefix == Meta);
+		    if (p > compprefix + l)
+			p = compprefix + l;
 		    add = 1;
 		}
 		for (;;) {
@@ -835,10 +835,9 @@ do_comp_vars(int test, int na, char *sa, int nb, char *sb, int mod)
 		    if (add > 0) {
 			if (p == compprefix + l)
 			    return 0;
-			if (*p == Meta)
-			    p += 2;
-			else
-			    p++;
+			p = p + 1 + (*p == Meta);
+			if (p > compprefix + l)
+			    p = compprefix + l;
 		    } else {
 			if (p == compprefix)
 			    return 0;
